{% macro trial_details(trial, source_file=None) %}
<div class="trial {% if trial.result.found %} found {% else %} not_found {% endif %}">
    <p><span class="label">Status:</span> {{ trial.result.reason_code }}</p>
    {% if source_file %}
    <p><span class="label">Source File:</span> {{ source_file }}</p>
    {% endif %}
    <p><span class="label">Mutator Name:</span> {{ trial.mutant.mutator_name }}</p>
    <p><span class="label">Unified Diff:</span></p>
    <pre>{{ trial.mutant.unified_diff }}</pre>
    {% if trial.result.reason_desc %}
    <p><span class="label">Description:</span></p>
    <pre>{{ trial.result.reason_desc }}</pre>
    {% endif %}
</div>
{% endmacro %}

<!DOCTYPE html>
<html>

<head>
    {% block head %}
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <title>{{ title }}</title>
    <link rel="stylesheet" type="text/css" href="html-report.css" />
    <link rel="icon" href="poodle.ico" type="image/x-icon" />
    {% endblock %}
</head>

<body>
    <header>
        {% block header %}
        <h1>{{ title }}</h1>
        {% if project.name %}
        <span class="subtitle"><span class="label">Project:</span> {{ project.name }}
            {% if project.version %} version {{ project.version }} {% endif %}</span>
        {% endif %}
        {% endblock %}
    </header>
    <main>
        {% block main %}
        {% endblock %}
    </main>
    <footer>
        {% block footer %}
        <p>Generated by <a href="http://poodle.rtfd.io" class="nav">Poodle</a> version {{ poodle_version }} at {{ timestamp }}</p>
        {% endblock %}
    </footer>
    {% block scripts %}
    <script type="text/javascript" src="html-report.js"></script>
    {% endblock %}
</body>

</html>